/*
* SessionManager.ts
* @author Abhilash Panwar (abpanwar) Haorui Guo (haoruiguo)
* @copyright Microsoft 2020
* File containing apis to help send SDK usage stats.
*/

import { IExtendedTelemetryItem, EventsDiscardedReason } from '@microsoft/1ds-core-js';
import { _Extensions, _SignalExtKeys, _StatsEventName, _StatsSchemaVersion, _eventSent, _eventDiscardedInvalidEvent, _eventDiscardedKillSwitch, _eventDiscardedNonRetryableStatus, _eventDiscardedQueueFull, _eventDiscardedSizeLimitExceeded, _eventDiscardedUnknown, _eventSendRequest, _eventDiscarded } from './Constants'

const StatsTimer = 60000;

export default class StatsManager {
    private _isInitalized = false;
    private _stats:  { [ingestionKey: string]: { [statName: string]: number } } = {};
    private _sendStats: (stats: { [statName: string]: number }, ingestionKey: string) => void;
    private _timeoutId: ReturnType<typeof setTimeout>


    /**
     * Creates an instance of the stats manager class to send SDK usage statistics.
     * @param sendStats stats will be sent to the tenant using this function.
     */
    constructor(sendStats: (stats: { [statName: string]: number }, ingestionKey: string) => void) {
        this._isInitalized = true;
        this._sendStats = sendStats;
        setTimeout(() => this._flush(), StatsTimer);
    }

    /**
     * Creates an event if there's stats and calls sendStats.
     */
    _flush() {
        if (this._isInitalized) {
            for (let ingestionKey in this._stats) {
                if (this._stats.hasOwnProperty(ingestionKey)) {
                    this._sendStats(this._stats[ingestionKey], ingestionKey);
                }
            }
            this._stats = {};
            this._timeoutId = setTimeout(() => this._flush(), StatsTimer);
        }
    }

    /**
     * Flush the current stats and stop the stats collection.
     */
    _teardown() {
        if (this._isInitalized) {
            this._flush();
            clearTimeout(this._timeoutId);
            this._isInitalized = false;
        }
    }

    /**
     * Add stats for events successfully sent from SDK.
     */
    _addStatForEventSent (events: IExtendedTelemetryItem[]) {
        events = this._getNonStatsEvents(events);
        if (events.length === 0) return;
        this._addStatForEvents(events, _eventSent);
    }

    /**
     * Add stats for events being requested to send but were discarded by SDK
     */
    _addStatForEventDiscarded (events: IExtendedTelemetryItem[], reason: number) {
        events = this._getNonStatsEvents(events);
        if (events.length === 0) return;
        this._addStatForEvents(events, _eventDiscarded);
        switch(reason) {
            case EventsDiscardedReason.InvalidEvent:
                this._addStatForEvents(events, _eventDiscardedInvalidEvent);
                break;
            case EventsDiscardedReason.KillSwitch:
                this._addStatForEvents(events, _eventDiscardedKillSwitch);
                break;
            case EventsDiscardedReason.NonRetryableStatus:
                this._addStatForEvents(events, _eventDiscardedNonRetryableStatus);
                break;
            case EventsDiscardedReason.QueueFull:
                this._addStatForEvents(events, _eventDiscardedQueueFull);
                break;
            case EventsDiscardedReason.SizeLimitExceeded:
                this._addStatForEvents(events, _eventDiscardedSizeLimitExceeded);
                break;
            case EventsDiscardedReason.Unknown:
                this._addStatForEvents(events, _eventDiscardedUnknown);
                break;
        }
    }

    private _addStatForEvents(events: IExtendedTelemetryItem[], stat: string) {
        for (let event of events) {
            this._addStat(stat, 1, event.iKey);
        }
    }

    private _addStat(statName: string, value: number, ingestionKey: string) {
        if (this._isInitalized) {
            if (!this._stats[ingestionKey]) {
                this._stats[ingestionKey] = {};
            }
            if (!this._stats[ingestionKey][statName]) {
                this._stats[ingestionKey][statName] = value;
            } else {
                this._stats[ingestionKey][statName] = this._stats[ingestionKey][statName] + value;
            }
        }
    }

    private _getNonStatsEvents (events: IExtendedTelemetryItem[]) {
        let nonStatsEvents: IExtendedTelemetryItem[] = [];
        for (let event of events) {
            if (event.name !== _StatsEventName) {
                nonStatsEvents.push(event);
            }
        }
        return nonStatsEvents;
    }
}